Attribute VB_Name = "DirectDrawHelper"
'DirectDraw - Basic Functionality Library for VB5.0
'Copyright Digital Zone Software (c) 1997

'Provides several functions to ease the use of
'Microsofts DirectDraw API
'This module is provided free of charge, and may
'be used and altered at will

'If you wish to redistribute this, please leave this
'header intact, and metion alterations made by you

'LEGAL MUMBO-JUMBO :
'This module is provided as is, and Digital Zone nor any
'of its employees will not take responsibility for any
'damages to hardware or software occuring as a result
'using this software.

'Other DirectX tutorials, modules and tools can be
'found on our website, http:\\www.mis.co.za\~johnh\directx.htm

'Any comments, suggestions etc regarding this module can
'be sent to lolasoft@ilink.nis.za
'Please give your message the subject DirectDraw Helper

'DDS refers to a DirectDrawSurface object

'KNOWN ISSUSES :
' (1) This module is intented for Full Screen
'     application only, and will not work if
'     Windowed is passed as a mode parameter to ddInit.

' (2) The dTextBk isn't used if ddTextMode is set to
'     Opaque. The cause is as yet unknown

' (3) The palette functions are not yet working

' (4) These routines perform very minimal error checking

'DirectDraw routine convention : ddRoutineName

'ROUTINES :
'  Initialization :
'    ddInit(frm, Resolution, Mode, ColorMode)
'    ddInitFullScreen(frm)   640x480 256 Full Screen
'    ddInitWindowed(frm)     640x480 256 Windowed
'    ddInitModeX(frm)        320x240 ModeX Full Screen
'    ddInitLowRes(frm)       320x200 256 Full Screen
'    ddInitTrueColor(frm)    640x480 24-bit Full Screen

'  Bitmap :
'    function ddLoadBitMap(bmpFile As String) as DDS
                'Load bitmap from bmpFile and returns surface

'  Palette :
'    function ddUsePalette(bmpFile As String) As DirectDrawPalette
'    function ddLoadPalette(palFile as string) as DirectDrawPalette

'  Text :
'    ddGDIWrite(Surface , X, Y, Msg as string)
'               Writes text using GDI
'    ddGDIWriteFont(Surface, X, Y, Msg, Font)
'               Writes text using GDI and font Font.
'               Font is a LONG created using CreateFont

'  Surface :
'    ddSetColorKey(Surface, ColorIndex)
'             Sets the source color key of Surface
'             For 256 color mode : Index is palette index
'                 16/24-bit mode : Index is RGB

'    function ddCreateSurface(Width, Height) as DDS
               'Create a blank surface of widthxHeight, returns surface created
               
'    function ddClearSurface(Surface)
               'clears the surface to black
'    ddGetSurfaceSize(Surface, Width, Height)
               'Fills Width, Height with Surface dimensions
               
'    ddBltFast(X,Y,Source)
               'bltfast of source onto back buffer with source color key
'    ddBltFastNoColorKey(X,Y,Source)
               'ditto but with no color key
               
'    ddFlip                    Flips the primary/back buffers

'  Mouse
'    HideMouse        Hide the mouse cursor
'    ShowMouse      Shows the mouse cursor

Option Explicit

'Screen Size consts
Public Enum displayModes
    m320x200
    m320x240
    m640x480
    m640x400
    m800x600
    m1024x768
End Enum

'Mode consts
Global Const FullScreen = DDSCL_FULLSCREEN Or DDSCL_EXCLUSIVE
Global Const Windowed = DDSCL_NORMAL
Global Const d3dFullScreen = FullScreen Or 1024
Global Const ModeX = FullScreen Or DDSCL_ALLOWMODEX

'Color depth consts
Public Enum pixelFormats
    c8_bit = 8
    c16_bit = 16
    c24_bit = 24
    
    c256 = 8
    c65_thousand = 16
    c16_million = 24
End Enum

'Error Consts
Global Const ddBmpNotFound = &HFFFF0000
Global Const ddSurfaceFailed = &HFFFF0001

'Text Consts
Global Const TRANSPARENT = 1
Global Const OPAQUE = 2

'BLTing consts
Global Const ddNoColorKey = DDBLTFAST_WAIT Or DDBLTFAST_NOCOLORKEY
Global Const ddSrcColorKey = DDBLTFAST_WAIT Or DDBLTFAST_SRCCOLORKEY
Global Const ddBltSprite = ddSrcColorKey

Global Const ddNoColorKeyWindowed = DDBLT_WAIT Or DDBLT_ROP
Global Const ddBltSpriteWindowed = DDBLT_KEYSRC Or ddNoColorKeyWindowed

Global Const ddOpaque = DDBLTFAST_NOCOLORKEY
Global Const ddTransparent = DDBLTFAST_SRCCOLORKEY

'PUBLIC INTERFACE OBJECTS

'Main DirectDraw object
Public dDraw As DirectDraw2

    'stores display resolution for easy ascess
    'in windowed mode size of client area
Public displayHRes As Long
Public displayVRes As Long

Private ddUseScaling As Boolean

Public xScale As Single
Public yScale As Single

Private xFull As Integer
Private yFull As Integer

    'current pixel format - use it to deterine color
    'mode when in windowed mode
    'possible values match those passed to ddInit
Public ddColorMode As Long

'DirectDraw Surfaces
Public PrimarySurface As DirectDrawSurface2
Public BackBuffer As DirectDrawSurface2
Public ddClipper As DirectDrawClipper

'GDI related vars
Public ddTextColor As Long          'color of ddWriteText
Public ddTextBk As Long
Public ddTextMode As Long

'Error condition variables
Public ddError As Long

Public ddOK As Boolean

Private ddsdFront As DDSURFACEDESC      ' Front surface description
Private DDCAPS As DDSCAPS               ' Capabilities for search
Public ddWindowed As Boolean           ' TRUE is running windowed
Private ddhWnd As Long                  ' Handle of owner form
Public ddWinBlt As DDBLTFX             ' fx object for windowed Blt

Const IMAGE_BITMAP = 0
Const LR_LOADFROMFILE = &H10
Const LR_CREATEDIBSECTION = &H2000
Const SRCCOPY = &HCC0020

Private Type BITMAP
        bmType          As Long
        bmWidth         As Long
        bmHeight        As Long
        bmWidthBytes    As Long
        bmPlanes        As Integer
        bmBitsPixel     As Integer
        bmBits          As Long
End Type

Private Type RGBQUAD
        rgbBlue As Byte
        rgbGreen As Byte
        rgbRed As Byte
        rgbReserved As Byte
End Type

Private Type BITMAPFILEHEADER
        bfType As Integer
        bfSize As Long
        bfReserved1 As Integer
        bfReserved2 As Integer
        bfOffBits As Long
End Type

Private Type BITMAPINFOHEADER '40 bytes
        biSize As Long
        biWidth As Long
        biHeight As Long
        biPlanes As Integer
        biBitCount As Integer
        biCompression As Long
        biSizeImage As Long
        biXPelsPerMeter As Long
        biYPelsPerMeter As Long
        biClrUsed As Long
        biClrImportant As Long
End Type

Declare Function lclose Lib "Kernel32" Alias "_lclose" (ByVal hFile As Long) As Long
Declare Function lopen Lib "Kernel32" Alias "_lopen" (ByVal lpPathName As String, ByVal iReadWrite As Long) As Long
Declare Function lread Lib "Kernel32" Alias "_lread" (ByVal hFile As Long, lpBuffer As Any, ByVal wBytes As Long) As Long
Public Const OF_READ = &H0

' GDI32 Functions
Private Declare Function GetPixel Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long) As Long
Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function GetObject Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, lpObject As Any) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function ClientToScreen Lib "user32" (ByVal hwnd As Long, lpPoint As POINTAPI) As Long
Private Declare Function GetClientRect Lib "user32" (ByVal hwnd As Long, lpRect As RECT) As Long

Type POINTAPI
        X As Long
        Y As Long
End Type

' USER32 Functions
Private Declare Function GetDC Lib "user32" (ByVal hwnd As Long) As Long
Private Declare Function LoadImage Lib "user32" Alias "LoadImageA" (ByVal hInst As Long, ByVal lpsz As String, ByVal un1 As Long, ByVal n1 As Long, ByVal n2 As Long, ByVal un2 As Long) As Long
Private Declare Function StretchBlt Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal nSrcWidth As Long, ByVal nSrcHeight As Long, ByVal dwRop As Long) As Long
Private Declare Function SetTextColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long
Private Declare Function TextOut Lib "gdi32" Alias "TextOutA" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal lpString As String, ByVal nCount As Long) As Long
Private Declare Function SetBkColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long
Private Declare Function SetBkMode Lib "gdi32" (ByVal hdc As Long, ByVal nBkMode As Long) As Long

Private Declare Sub CopyMemory Lib "Kernel32" Alias "RtlCopyMemory" (hpvDest As Any, hpvSource As Any, ByVal cbCopy As Long)
Private Declare Sub FillMemory Lib "Kernerl32" Alias "RtlFillMemory" (ByVal l As Long, ByVal length As Long, ByVal Fill As Byte)
Private Declare Function CreateFont Lib "gdi32" Alias "CreateFontA" (ByVal h As Long, ByVal W As Long, ByVal E As Long, ByVal O As Long, ByVal W As Long, ByVal i As Long, ByVal u As Long, ByVal S As Long, ByVal C As Long, ByVal OP As Long, ByVal CP As Long, ByVal Q As Long, ByVal PAF As Long, ByVal F As String) As Long
Private Declare Function GetSystemPaletteEntries Lib "gdi32" (ByVal hdc As Long, ByVal First As Long, ByVal Size As Long, ptr As Any) As Long
    
Private Declare Function ShowCursor Lib "user32" (ByVal bShow As Long) As Long

Public Sub ddSetDefaultScale(ByVal fX As Integer, ByVal fY As Integer)
 xFull = fX
 yFull = fY
End Sub

Private Function ddiWindowed(ByVal hwnd As Long) As Boolean
 Dim wndSize As RECT
 
 ddiWindowed = True

 With ddsdFront
  .dwSize = Len(ddsdFront)
  .dwFlags = DDSD_CAPS
  .DDSCAPS.dwCaps = DDSCAPS_PRIMARYSURFACE
 End With
 
 ' The primary surface is not a page flipping surface this time
 ' Surface will be size of desktop
 dDraw.CreateSurface ddsdFront, PrimarySurface, Nothing

 ' Create a clipper to ensure that our drawing stays inside our window
 dDraw.CreateClipper 0, ddClipper, Nothing
 'setting it to our hwnd gives the clipper the coordinates from our window
 ddClipper.SetHWnd 0, hwnd
 'attach the clipper to the primary surface
 PrimarySurface.SetClipper ddClipper
        
 'back buffer is a normal surface
 With ddsdFront
    .dwSize = Len(ddsdFront)
    .dwFlags = DDSD_CAPS Or DDSD_HEIGHT Or DDSD_WIDTH
    .DDSCAPS.dwCaps = DDSCAPS_OFFSCREENPLAIN Or DDSCAPS_SYSTEMMEMORY
    
        'backbuffer is size of client with overlap to allow
        'for integer scaling
    GetClientRect hwnd, wndSize
    .dwWidth = wndSize.Right + 32
    .dwHeight = wndSize.bottom + 32
 End With
        
 'create the backbuffer separately
 dDraw.CreateSurface ddsdFront, BackBuffer, Nothing
  
 With ddsdFront
  .dwSize = Len(ddsdFront)
  .dwFlags = DDSD_PIXELFORMAT Or DDSD_WIDTH Or DDSD_HEIGHT
 End With
 dDraw.GetDisplayMode ddsdFront
 
 With ddsdFront.ddpfPixelFormat
  If .dwFlags And DDPF_RGB Then
    ddColorMode = .dwRGBBitCount
  Else
   If .dwFlags And DDPF_PALETTEINDEXED8 Then
    ddColorMode = 8
   Else
     'report error - 16 color/mono modes not supported
   End If
  End If
 End With
   
    'display size is restricted to window size
 displayVRes = wndSize.bottom
 displayHRes = wndSize.Right
 
 'xFull = 640
 'yFull = 480
 
 ddWinBlt.dwSize = Len(ddWinBlt)
 ddWinBlt.dwRop = SRCCOPY

 ddWindowed = True
 ddiWindowed = True
End Function
                                                                
Private Function ddiFullScreen(ByVal hwnd As Long, ByVal ScreenRes As displayModes, ByVal ColorMode As pixelFormats, ByVal addCaps As Long) As Boolean
 Dim dmFlags As Long

  ddiFullScreen = True
  
  ' Set the display mode
  dmFlags = 0
  Select Case ScreenRes
   Case m320x200
            displayHRes = 320
            displayVRes = 200
            dmFlags = 1
   Case m320x240
            displayHRes = 320
            displayVRes = 240
   Case m640x400
            displayHRes = 640
            displayVRes = 400
   Case m640x480
            displayHRes = 640
            displayVRes = 480
   Case m800x600
            displayHRes = 800
            displayVRes = 600
   Case m1024x768
            displayHRes = 1024
            displayVRes = 768
  End Select
   
  dDraw.SetDisplayMode displayHRes, displayVRes, ColorMode, 0, dmFlags
     
  ' Fill front buffer description structure...
  With ddsdFront
    ' Structure size
    .dwSize = Len(ddsdFront)
    ' Use DDSD_CAPS and BackBufferCount
    .dwFlags = DDSD_CAPS Or DDSD_BACKBUFFERCOUNT
    ' Primary, flipable surface
    .DDSCAPS.dwCaps = DDSCAPS_PRIMARYSURFACE Or DDSCAPS_FLIP Or DDSCAPS_COMPLEX Or DDSCAPS_SYSTEMMEMORY Or addCaps
    ' One back buffer (you can try 2)
    .dwBackBufferCount = 1
  End With
    
  ' Create front buffer
  dDraw.CreateSurface ddsdFront, PrimarySurface, Nothing
    
  ' Retrieve the back buffer object
  DDCAPS.dwCaps = DDSCAPS_BACKBUFFER
  PrimarySurface.GetAttachedSurface DDCAPS, BackBuffer
  
  ddColorMode = ColorMode
     
  ddWindowed = False
  ddiFullScreen = True
End Function

'Direct Draw init with all params
Sub ddInit(frm As Form, ByVal ScreenRes As displayModes, ByVal Mode As Long, ByVal ColorMode As pixelFormats)
  
  ' Create the DirectDraw object
  DirectDrawCreate ByVal 0&, dDraw, Nothing
  
  dDraw.SetCooperativeLevel frm.hwnd, Mode And 1023
  
  Select Case Mode
   Case FullScreen:  ddiFullScreen frm.hwnd, ScreenRes, ColorMode, 0
   Case Windowed: ddiWindowed frm.hwnd
   Case d3dFullScreen: ddiFullScreen frm.hwnd, ScreenRes, ColorMode, DDSCAPS_3DDEVICE
  End Select
  
  ddhWnd = frm.hwnd
        
        'if no full scaling size has been set,
        'defualt it to the display res or window size
  If (xFull And yFull) = 0 Then ddSetDefaultScale displayHRes, displayVRes
   
  xScale = displayHRes / xFull
  yScale = displayVRes / yFull
   
  ddUseScaling = ddWindowed Or ((xScale <> 1) Or (yScale <> 1))
   
  ddTextColor = RGB(255, 255, 255)
  ddTextBk = RGB(0, 0, 0)
  ddTextMode = TRANSPARENT
  
  ddOK = True
End Sub


'Standard mode initialization
'Sets up a Full Screen 640x480 256 colour mode
Sub ddInitFullScreen(frm As Form)
 
 ddInit frm, m640x480, FullScreen, c256

End Sub

'Standard windowed mode initialization
'Sets up  a Windowed 640x480 256 color mode
Sub ddInitWindowed(frm As Form)

 ddInit frm, m640x480, Windowed, c256

End Sub

'Sets up 320x240 ModeX Full Screen
Sub ddInitModeX(frm As Form)
 
  ddInit frm, m320x240, ModeX, c256
  
End Sub

'Sets up 640x480 24-bit mode
Sub ddInitTrueColor(frm As Form)
 
  ddInit frm, m640x480, FullScreen, c16_million
  
End Sub

'Sets up 320x200 'standard' VGA low res mode
Sub ddInitLowRes(frm As Form)
 
 ddInit frm, m320x200, FullScreen, c256

End Sub

Sub ddUnload()
  
  If ddOK Then
   If Not ddWindowed Then
    dDraw.FlipToGDISurface
    dDraw.RestoreDisplayMode
    dDraw.SetCooperativeLevel 0, DDSCL_NORMAL
   Else
    Set ddClipper = Nothing
   End If
     
   Set BackBuffer = Nothing
   Set PrimarySurface = Nothing
   Set dDraw = Nothing
   
   ShowMouse
  End If

End Sub

Public Sub HideMouse()
 ShowCursor 0
End Sub

Public Sub ShowMouse()
 ShowCursor 1
End Sub

' Loads a bitmap in a DirectDraw surface
 Function ddLoadBitmap(ByVal strFile As String, Optional fromRes) As DirectDrawSurface2
    Dim hbm As Long                 ' Handle on bitmap
    Dim bm As BITMAP                ' Bitmap header
    Dim ddsd As DDSURFACEDESC       ' Surface description
    Dim dds As DirectDrawSurface2   ' Created surface
    Dim hdcImage As Long            ' Handle on image
    Dim mhdc As Long                ' Handle on surface context
    Dim clr As Long                 ' hold the colour top left to be made transparent
    Dim FoundIt As Boolean
    
    FoundIt = False
    If (IsMissing(fromRes)) Then
     FoundIt = (Dir(strFile, vbNormal) <> "")
     If FoundIt Then hbm = LoadImage(ByVal 0&, strFile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE Or LR_CREATEDIBSECTION)
    Else
     If fromRes Then
      hbm = LoadResPicture(strFile, vbResBitmap)
      FoundIt = True
     Else
      FoundIt = (Dir(strFile, vbNormal) <> "")
      If FoundIt Then hbm = LoadImage(ByVal 0&, strFile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE Or LR_CREATEDIBSECTION)
     End If
    End If
        
    ' Load bitmap
    If FoundIt Then 'Dir(strFile, vbNormal) <> "" Then
        
        'hbm = LoadImage(ByVal 0&, strFile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE Or LR_CREATEDIBSECTION)
        
        ' Get bitmap info
        GetObject hbm, Len(bm), bm
        ' Fill surface description
        With ddsd
            .dwSize = Len(ddsd)
            .dwFlags = DDSD_CAPS Or DDSD_HEIGHT Or DDSD_WIDTH
            .DDSCAPS.dwCaps = DDSCAPS_OFFSCREENPLAIN
            .dwWidth = bm.bmWidth
            .dwHeight = bm.bmHeight
        End With
        ' Create surface
        dDraw.CreateSurface ddsd, dds, Nothing
        ' Create memory device
        hdcImage = CreateCompatibleDC(ByVal 0&)
        ' Select the bitmap in this memory device
        SelectObject hdcImage, hbm
        ' Restore the surface
        dds.Restore
        ' Get the surface's DC
        dds.GetDC mhdc
        ' Copy from the memory device to the DirectDrawSurface
        StretchBlt mhdc, 0, 0, ddsd.dwWidth, ddsd.dwHeight, hdcImage, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY
        'get the top left colour
        clr = GetPixel(mhdc, 0, 0)
        ' Release the surface's DC
        dds.ReleaseDC mhdc
        ' Release the memory device and the bitmap
        DeleteDC hdcImage
        DeleteObject hbm
        
        'make surface transparent
        Dim mhddck As DDCOLORKEY
        mhddck.dwColorSpaceLowValue = 0 'clr
        mhddck.dwColorSpaceHighValue = 0 'clr
        dds.SetColorKey DDCKEY_SRCBLT, mhddck
        
               
        ' Returns the new surface
        Set ddLoadBitmap = dds
        ddError = DD_OK
   Else
     
     Set ddLoadBitmap = Nothing
     ddError = ddBmpNotFound
     
   End If
End Function

Public Function ddUsePalette(bmpFile As String) As DirectDrawPalette
 On Error GoTo usepalError
 
 Dim palFile As Long
 Dim ddp As DirectDrawPalette
 Dim aPalette(256) As RGBQUAD
 Dim palEntries(256) As PALETTEENTRY
 Dim bi As BITMAPINFOHEADER
 Dim bf As BITMAPFILEHEADER
 Dim i As Long
  
' palFile = FreeFile
' Open App.Path + "\dz.pal" For Binary Access Read As #palFile
 'Get #palFile, , bf
 'Get #palFile, , bi
' Get #palFile, 17, Palette
 'Close #palFile
 palFile = lopen(bmpFile, OF_READ)
 lread palFile, bf, Len(bf)
 lread palFile, bi, Len(bi)
 lread palFile, aPalette(0), 255 * 4
 lclose palFile
 
 Dim t As Byte
 For i = 0 To 255
   palEntries(i).peRed = aPalette(i).rgbRed
   palEntries(i).peGreen = aPalette(i).rgbGreen
   palEntries(i).peBlue = aPalette(i).rgbBlue
   palEntries(i).peFlags = 0

 ' t = palEntries(i).peBlue
  'palEntries(i).peBlue = palEntries(i).peGreen '= aPalette(i).rgbRed
 ' 'palEntries(i).pegreen = aPalette(i).rgbGreen
 ' palEntries(i).peGreen = t ' aPalette(i).rgbBlue
  'palEntries(i).peFlags = 0
 Next i
 
 dDraw.CreatePalette DDPCAPS_8BIT, palEntries(0), ddp, Nothing
 PrimarySurface.SetPalette ddp
 Set ddUsePalette = ddp
 Exit Function
 
usepalError:
  Set ddUsePalette = Nothing
End Function

'Sets the source color key of a surface
'Limited to only one key
'for 256 color : index is palettte entry,
'    16/24-bit : index is RGB
Public Sub ddSetColorKey(Surface As DirectDrawSurface2, ByVal index As Long)
 Dim colorKey As DDCOLORKEY
 colorKey.dwColorSpaceHighValue = index
 colorKey.dwColorSpaceLowValue = index
 Surface.SetColorKey DDCKEY_SRCBLT, colorKey
End Sub


'Returns the Width, Height of a surface in Width,Height.
Public Sub ddGetSurfaceSize(Surface As DirectDrawSurface2, Width As Long, Height As Long)
  Dim sDesc As DDSURFACEDESC
  
  sDesc.dwSize = Len(sDesc)
  sDesc.dwFlags = DDSD_HEIGHT Or DDSD_WIDTH
  Surface.GetSurfaceDesc sDesc
  
  Width = sDesc.dwWidth
  Height = sDesc.dwHeight
End Sub

'Simple blit routine.
'Always blits to BackBuffer, always BltFast
'Have to use whole source surface
'Always blts using source color key, use ddBltFastNoColorKey for no color keying
Public Sub ddBltFast(ByVal X As Integer, ByVal Y As Integer, source As DirectDrawSurface2)
  Dim t As RECT
  Dim Src As RECT
  Dim sDesc As DDSURFACEDESC
  
  sDesc.dwSize = Len(sDesc)
  sDesc.dwFlags = DDSD_HEIGHT Or DDSD_WIDTH
  source.GetSurfaceDesc sDesc
  
  t.Top = 0
  t.Left = 0
  t.bottom = sDesc.dwHeight
  t.Right = sDesc.dwWidth
 
  If Not ddWindowed Then
   BackBuffer.BltFast X, Y, source, t, ddSrcColorKey
  Else
   Src.Top = Y
   Src.Left = X
   Src.Right = X + sDesc.dwWidth
   Src.bottom = Y + sDesc.dwHeight
   
   BackBuffer.Blt Src, source, t, DDBLT_ROP Or DDBLT_WAIT Or DDBLT_KEYSRC, ddWinBlt
  End If
End Sub

Public Sub ddBltFastNoColorKey(ByVal X As Integer, ByVal Y As Integer, source As DirectDrawSurface2)
  Dim t As RECT
  Dim Src As RECT
  Dim sDesc As DDSURFACEDESC
  
  sDesc.dwSize = Len(sDesc)
  sDesc.dwFlags = DDSD_HEIGHT Or DDSD_WIDTH
  source.GetSurfaceDesc sDesc
  
  t.Top = 0
  t.Left = 0
  t.bottom = sDesc.dwHeight
  t.Right = sDesc.dwWidth
  
 If Not ddWindowed Then
   BackBuffer.BltFast X, Y, source, t, ddNoColorKey
 Else
   Src.Top = Y
   Src.Left = X
   Src.Right = X + sDesc.dwWidth
   Src.bottom = Y + sDesc.dwHeight

   BackBuffer.Blt Src, source, t, DDBLT_WAIT Or DDBLT_ROP, ddWinBlt
  End If
End Sub

Public Sub ddBlt(ByVal X As Integer, ByVal Y As Integer, Surface As DirectDrawSurface2, clipRect As RECT, ByVal Flags As Long)
 If ddUseScaling Then
  Dim Dest As RECT
  Dest.Left = X * xScale
  Dest.Top = Y * yScale
  Dest.bottom = Dest.Top + CInt((clipRect.bottom - clipRect.Top) * yScale)
  Dest.Right = Dest.Left + CInt((clipRect.Right - clipRect.Left) * xScale)
  
  If Flags = ddTransparent Then
   Flags = DDBLT_KEYSRC
  Else
   Flags = 0
  End If
  
  BackBuffer.Blt Dest, Surface, clipRect, Flags Or DDBLT_ROP Or DDBLT_WAIT, ddWinBlt
 Else
  BackBuffer.BltFast X, Y, Surface, clipRect, Flags Or DDBLTFAST_WAIT
 End If
End Sub

'Creates a blank surface of dimensions fW(width) x fH(height)
'No color key, surface is black
Public Function ddCreateSurface(ByVal fW As Long, ByVal fH As Long)
 Dim ddsd As DDSURFACEDESC
 Dim fX As DDBLTFX
 Dim ts As DirectDrawSurface2
 Dim t As RECT
 
 With ddsd
  .dwSize = Len(ddsd)
  .dwFlags = DDSD_CAPS Or DDSD_HEIGHT Or DDSD_WIDTH
  .DDSCAPS.dwCaps = DDSCAPS_OFFSCREENPLAIN
  .dwWidth = fW
  .dwHeight = fH
 End With
        
 ' Create surface
  On Error Resume Next
  dDraw.CreateSurface ddsd, ts, Nothing
  If Err = DD_OK Then
  
   ' Clear the surface
   With fX
    .dwSize = Len(fX)
    .dwFillColor = RGB(0, 0, 0)
   End With
   t.Top = 0
   t.Left = 0
   t.bottom = fH
   t.Right = fW
   ts.Blt t, Nothing, t, DDBLT_COLORFILL, fX
   
   Set ddCreateSurface = ts
  Else
   Set ddCreateSurface = Nothing
   ddError = Err
  End If
 
End Function

'Uses the GDI to write text to a surface in ddTextColor
'ddBkColor is text background if ddTextMode = Opaque
Public Sub ddGDIWrite(Surface As DirectDrawSurface, ByVal X As Integer, ByVal Y As Integer, Msg As String)
  On Error GoTo gdiWriteError
  
  Dim dsDC As Long
  
  Surface.GetDC dsDC
  
  SetBkMode dsDC, ddTextMode
  SetBkColor dsDC, ddTextBk
  SetTextColor dsDC, ddTextColor
  
  TextOut dsDC, X, Y, Msg, Len(Msg)
  
  Surface.ReleaseDC dsDC
  
gdiWriteError: Debug.Print "GDI ERROR WRITING " + Msg
End Sub

Public Sub ddGDIWriteFont(Surface As DirectDrawSurface, ByVal X As Integer, ByVal Y As Integer, Msg As String, aFont As Long)
  On Error GoTo gdiWriteError
  
  Dim dsDC As Long
  Dim OldFont As Long
  
  Surface.GetDC dsDC
  
  SetBkMode dsDC, ddTextMode
  SetBkColor dsDC, ddTextBk
  SetTextColor dsDC, ddTextColor
 
  SelectObject dsDC, aFont
 
  TextOut dsDC, X, Y, Msg, Len(Msg)
  
  Surface.ReleaseDC dsDC
  
gdiWriteError: Debug.Print "GDI ERROR WRITING " + Msg
 
End Sub

'Clears the surface to black
Public Sub ddClearSurface(Surface As DirectDrawSurface2)
 Dim fX As DDBLTFX
 Dim t As RECT
 Dim fW As Long, fH As Long
 With fX
  .dwSize = Len(fX)
  .dwFillColor = RGB(0, 0, 0)
 End With
   
 ddGetSurfaceSize Surface, fW, fH
 t.Top = 0
 t.Left = 0
 t.bottom = fH
 t.Right = fW
 Surface.Blt t, Nothing, t, DDBLT_COLORFILL, fX

End Sub
'Flips the primary and back buffers.
'Loops until the flip is succesful
Public Sub ddFlip()
 Dim p As POINTAPI
 Dim Dest As RECT
 Dim Src As RECT
 ddError = 0
 
 If ddWindowed Then
   'Get current co-ords of our window on the primary surface
   p.X = 0
   p.Y = 0
   ClientToScreen ddhWnd, p
   
        'get the rect containing our window
   GetClientRect ddhWnd, Dest
   
        'use the whole back buffer
   Src = Dest
   Src.Left = 0
   Src.Top = 0
   
        'offset the rectangle onto the primary surface
   Dest.Left = Dest.Left + p.X
   Dest.Right = Dest.Right + p.X
   Dest.Top = Dest.Top + p.Y
   Dest.bottom = Dest.bottom + p.Y
   
   PrimarySurface.Blt Dest, BackBuffer, Src, DDBLT_WAIT Or DDBLT_ROP, ddWinBlt
  
 Else
  Do
   PrimarySurface.Flip Nothing, 0
  
    If Err = DDERR_SURFACELOST Then
      PrimarySurface.Restore
      ddError = DDERR_SURFACELOST
    End If
  
  Loop Until Err = 0
 End If
End Sub

Public Sub ddOnResize()
 Dim wndSize As RECT
 
 If ddOK And ddWindowed Then
    Set BackBuffer = Nothing
 
    'back buffer is a normal surface
    With ddsdFront
     .dwSize = Len(ddsdFront)
     .dwFlags = DDSD_CAPS Or DDSD_HEIGHT Or DDSD_WIDTH
     .DDSCAPS.dwCaps = DDSCAPS_OFFSCREENPLAIN Or DDSCAPS_SYSTEMMEMORY
    
     GetClientRect ddhWnd, wndSize
     .dwWidth = wndSize.Right + 32
     .dwHeight = wndSize.bottom + 32
    End With
        
    'create the backbuffer separately
    dDraw.CreateSurface ddsdFront, BackBuffer, Nothing
   ' BackBuffer.SetClipper ddClipper
    
    displayVRes = wndSize.bottom
    displayHRes = wndSize.Right

    xScale = displayHRes / xFull
    yScale = displayVRes / yFull
 End If
 
End Sub

Public Function ddPointInRect(ByVal X As Integer, ByVal Y As Integer, R As RECT)
 ddPointInRect = False
 
 With R
  If (Y > .Top) And (Y < .bottom) Then
   ddPointInRect = (X > .Left) And (X < .Right)
  End If
 End With
End Function
