Attribute VB_Name = "DirectSoundHelper"
'Direct Sound Helper Module
'Provides several functions to ease the use of
'Microsofts DirectSound API and WinMM Midi
'This module is provided free of charge, and may
'be used and altered at will

'If you wish to distribute this, please leave this
'header intact, and metion alterations made by you

'LEGAL MUMBO-JUMBO :
'This module is provided as is, and Digital Zone
'will not take responsibility for any damages to
'hardware or software occuring as a result using this
'software.

'Other DirectX tutorials, modules and tools can be
'found on our website, http:\\www.mis.co.za\~johnh\directx.htm

'DSB refers to a DirectSoundBuffer object

'dSound is the main DirectSound object
'
'All DirectSound routines begin ds...
'Routines :
'   dsInit(Owning form) - initialize
'   function dsLoadWave(FileName) - creates a DSB
'         and loads the wave file into it, and
'         returns a DSB
'   dsPlay( DSB )  -  play the buffer
'   dsPlayLooped( DSB) - play looping forever
'   dsStop( DSB ) - Stops the buffer
'   dsStopCurrent( DSB ) - Stop the last buffer played with dsPlay...
'   dsGetVolume( DSB ) - returns buffers volume
'   dsRewind( DSB ) - sets the current position to the start
'   function dsDuplicate( DSB ) - duplicates the
'          DSB and returns the new DSB
'   dsToggleSound - toggle the dsSoundOn value

Option Explicit

Private Declare Sub CopyMemory Lib "Kernel32" Alias "RtlMoveMemory" (ByVal Destination As Long, ByVal source As Long, ByVal length As Long)
Private Declare Function lstrcpy Lib "Kernel32" (ByVal lpszDestinationString1 As Any, ByVal lpszSourceString2 As Any) As Long

Public dSound As DirectSound
Public dsSoundOn As Boolean

Private lastPlayed As DirectSoundBuffer

'Init a Direct Sound object with NORMAL co-operation level
Public Sub dsInit(ByVal h As Form)
   
   DirectSoundCreate ByVal 0&, dSound, Nothing
   dSound.SetCooperativeLevel h.hwnd, DSSCL_NORMAL
   dsSoundOn = True
   
End Sub

Public Sub dsUnload()
 Set lastPlayed = Nothing
 Set dSound = Nothing
End Sub

' Creates a DirectSoundBuffer from a wave file
Public Function dsLoadWave(ByVal strFile As String) As DirectSoundBuffer
    Dim dsb As DirectSoundBuffer
    Dim hWave As Long
    Dim pcmwave As WAVEFORMATEX
    Dim lngSize As Long
    Dim lngPosition As Long
    Dim ptr1 As Long, ptr2 As Long, lng1 As Long, lng2 As Long
    Dim aByte() As Byte
    
    'Dim lngBufferSize As Long
    'Dim strBuffer As String
    
    If InStr(1, strFile, ".wav", 1) = 0 Then
     strFile = strFile + ".wav"
    End If
    
    ' Byte array to load the whole file
    ReDim aByte(1 To FileLen(strFile))
    
    hWave = FreeFile
    Open strFile For Binary As hWave
    ' Load the whole file in the byte array
    Get hWave, , aByte
    Close hWave
    
    ' Search "fmt" tag
    lngPosition = 1
    While Chr$(aByte(lngPosition)) + Chr$(aByte(lngPosition + 1)) + Chr$(aByte(lngPosition + 2)) <> "fmt"
        lngPosition = lngPosition + 1
    Wend
    
    ' Copy wave header to structure
    CopyMemory VarPtr(pcmwave), VarPtr(aByte(lngPosition + 8)), Len(pcmwave)
    
    ' Search "data" tag
    While Chr$(aByte(lngPosition)) + Chr$(aByte(lngPosition + 1)) + Chr$(aByte(lngPosition + 2)) + Chr$(aByte(lngPosition + 3)) <> "data"
        lngPosition = lngPosition + 1
    Wend
    
    ' Get the data size
    CopyMemory VarPtr(lngSize), VarPtr(aByte(lngPosition + 4)), Len(lngSize)
    
    ' Fill buffer description
    Dim dsbd As DSBUFFERDESC
    With dsbd
        .dwSize = Len(dsbd)
        .dwFlags = DSBCAPS_CTRLDEFAULT 'Or DSBCAPS_STATIC Or DSBCAPS_LOCSOFTWARE
        .dwBufferBytes = lngSize
        .lpwfxFormat = VarPtr(pcmwave)
    End With
    
    ' Create the sound buffer
    dSound.CreateSoundBuffer dsbd, dsb, Nothing
    ' Lock
    dsb.Lock 0&, lngSize, ptr1, lng1, ptr2, lng2, 0&
    ' Copy data to buffer
    CopyMemory ptr1, VarPtr(aByte(lngPosition + 4 + 4)), lng1
    ' Copy second part if needed
    If lng2 <> 0 Then
        CopyMemory ptr2, VarPtr(aByte(lngPosition + 4 + 4 + lng1)), lng2
    End If
    ' Unlock
    ' Automation error if uncommented !
    'dsb.Unlock ptr1, lng1, ptr2, lng2
    Set dsLoadWave = dsb
End Function

Public Sub dsPlay(dsb As DirectSoundBuffer)
 If dsSoundOn Then
  dsb.Play 0, 0, 0
  Set lastPlayed = dsb
 End If
End Sub

Public Sub dsPlayLooped(dsb As DirectSoundBuffer)
 If dsSoundOn Then
  dsb.Play 0, 0, DSBPLAY_LOOPING
  Set lastPlayed = dsb
 End If
End Sub

Public Sub dsStop(dsb As DirectSoundBuffer)
  dsb.Stop
End Sub

Public Sub dsRewind(dsb As DirectSoundBuffer)
 dsb.SetCurrentPosition 0
End Sub

Public Function dsGetVolume(dsb As DirectSoundBuffer) As Long
 Dim V As Long
 dsb.GetVolume V
 dsGetVolume = V
End Function

Public Function dsDuplicate(dsb As DirectSoundBuffer) As DirectSoundBuffer
 Dim dsbDup As DirectSoundBuffer
 dSound.DuplicateSoundBuffer dsb, dsbDup
 Set dsDuplicate = dsbDup
End Function

Public Sub dsStopCurrent()
 lastPlayed.Stop
End Sub

Public Function dsIsPlaying(dsb As DirectSoundBuffer) As Boolean
 Dim S As Long
 
 dsb.GetStatus S
 dsIsPlaying = (S And DSBSTATUS_PLAYING) = DSBSTATUS_PLAYING
End Function

Public Sub dsToggleSound()
 dsSoundOn = Not dsSoundOn
End Sub


